@@ -5,7 +5,7 @@
 * pgpool: a language independent connection pool server for PostgreSQL
 * written by Tatsuo Ishii
 *
-* Copyright (c) 2003-2013	PgPool Global Development Group
+* Copyright (c) 2003-2015	PgPool Global Development Group
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for any purpose and without fee is hereby
@@ -79,6 +79,7 @@ POOL_CONNECTION *pool_open(int fd)
 	if (cp->hp == NULL)
 	{
 		pool_error("pool_open: malloc failed");
+		free(cp);
 		return NULL;
 	}
 	cp->bufsz = READBUFSZ;
@@ -88,6 +89,8 @@ POOL_CONNECTION *pool_open(int fd)
 	cp->sbufsz = 0;
 	cp->buf2 = NULL;
 	cp->bufsz2 = 0;
+	cp->buf3 = NULL;
+	cp->bufsz3 = 0;
 
 	cp->fd = fd;
 	return cp;
@@ -111,6 +114,8 @@ void pool_close(POOL_CONNECTION *cp)
 		free(cp->sbuf);
 	if (cp->buf2)
 		free(cp->buf2);
+	if (cp->buf3)
+		free(cp->buf3);
 	pool_discard_params(&cp->params);
 
 	pool_ssl_close(cp);
@@ -878,8 +883,8 @@ int pool_push(POOL_CONNECTION *cp, void *data, int len)
 	}
 	else
 	{
-		p = cp->buf3 + cp->bufsz3;
 		cp->buf3 = realloc(cp->buf3, cp->bufsz3 + len);
+		p = cp->buf3 + cp->bufsz3;
 	}
 
 	memcpy(p, data, len);
